"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var commands_1 = require("@codemirror/commands");
var lang_markdown_1 = require("@codemirror/lang-markdown");
var language_1 = require("@codemirror/language");
var view_1 = require("@codemirror/view");
var codemirror_theme_github_1 = require("@uiw/codemirror-theme-github");
var auto_1 = __importDefault(require("chart.js/auto"));
var debounce_1 = __importDefault(require("debounce"));
var manate_1 = require("manate");
var react_1 = require("manate/react");
var markdown_extensible_1 = __importDefault(require("markdown-extensible"));
var mermaid_esm_mjs_1 = __importDefault(require("mermaid/dist/mermaid.esm.mjs"));
var react_2 = __importStar(require("react"));
var styles = __importStar(require("../css/index.module.scss"));
var sync_scroll_1 = require("../sync-scroll");
var Editor = function (props) {
    var store = props.store;
    var editorDiv = (0, react_2.useRef)(null);
    (0, react_2.useEffect)(function () {
        var contentChangeListener = view_1.EditorView.updateListener.of(function (update) {
            if (update.docChanged) {
                lazyChange();
            }
        });
        // Define the custom key binding
        var customKeymap = view_1.keymap.of([
            {
                key: 'Tab',
                run: commands_1.indentMore,
            },
            {
                key: 'Shift-Tab',
                run: commands_1.indentLess,
            },
            {
                key: 'Mod-s',
                run: function () { return true; },
            },
            {
                key: 'Mod-b',
                run: function () {
                    var _a;
                    (_a = document
                        .querySelector(".".concat(styles['markplus'], " .toolbar i.fa-bold"))) === null || _a === void 0 ? void 0 : _a.click();
                    return true;
                },
            },
        ]);
        var cm = new view_1.EditorView({
            extensions: [
                store.editorTheme.of(codemirror_theme_github_1.githubLight),
                view_1.EditorView.lineWrapping,
                (0, view_1.highlightActiveLine)(),
                (0, commands_1.history)(),
                view_1.keymap.of(__spreadArray(__spreadArray([], commands_1.defaultKeymap, true), commands_1.historyKeymap, true)), // todo: type casting is unnecessary in the future
                (0, lang_markdown_1.markdown)(),
                (0, language_1.syntaxHighlighting)(language_1.defaultHighlightStyle),
                customKeymap,
                contentChangeListener,
            ],
            parent: editorDiv.current,
        });
        // auto focus after change text
        var dispatch = cm.dispatch.bind(cm);
        cm.dispatch = function (tr) {
            dispatch(tr);
            if (tr.changes && tr.changes.insert) {
                cm.focus();
            }
        };
        store.editor = (0, manate_1.exclude)(cm);
        var handleEditorScroll = function () {
            (0, sync_scroll_1.syncPreview)();
        };
        store.editor.scrollDOM.addEventListener('scroll', handleEditorScroll);
        var handlePreviewScroll = function () {
            (0, sync_scroll_1.syncEditor)();
        };
        var rightPanel = document.querySelector(".".concat(styles['markplus'], " .right-panel"));
        rightPanel.addEventListener('scroll', handlePreviewScroll);
        // whenever user changes markdown
        mermaid_esm_mjs_1.default.initialize({ startOnLoad: false, theme: 'neutral' });
        var lazyChange = (0, debounce_1.default)(function () {
            var markdown = store.editor.state.doc.toString();
            document.querySelector(".".concat(styles['markplus'], " .right-panel .preview")).innerHTML = markdown_extensible_1.default.render(markdown);
            // Chart.js
            document
                .querySelectorAll(".".concat(styles['markplus'], " .right-panel .preview .chartjs"))
                .forEach(function (element) {
                try {
                    new auto_1.default(element, JSON.parse(element.textContent));
                }
                catch (e) {
                    element.outerHTML = "<pre>Chart.js complains: \"".concat(e, "\"</pre>");
                }
            });
            // mermaid
            mermaid_esm_mjs_1.default.run({
                nodes: document.querySelectorAll(".".concat(styles['markplus'], " .right-panel .preview pre.mermaid")),
            });
        }, 256);
        return function () {
            store.editor.scrollDOM.removeEventListener('scroll', handleEditorScroll);
            store.editor.destroy();
            rightPanel.removeEventListener('scroll', handlePreviewScroll);
        };
    }, [store]);
    return react_2.default.createElement("div", { className: "editor", ref: editorDiv });
};
exports.default = (0, react_1.auto)(Editor);
//# sourceMappingURL=editor.js.map